/*
   Copyright 2009 senju@users.sourceforge.jp

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package jp.sourceforge.rabbitBTS.interceptors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.sourceforge.rabbitBTS.Sht;
import jp.sourceforge.rabbitBTS.controllers.IController;

import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

/**
 * CSRF対策用インターセプター
 */
public class CSRFInterceptor extends HandlerInterceptorAdapter {

	/**
	 * POSTの場合チェック処理
	 */
	@Override
	public boolean preHandle(HttpServletRequest request,
			HttpServletResponse response, Object handler) throws Exception {
		if (request.getMethod().equals("POST")
				&& handler instanceof IController) {
			final String sentToken = request.getParameter("secureToken");
			final String sessionToken = (String) request.getSession()
					.getAttribute("secureToken");
			final IController c = (IController) handler;

			if (StringUtils.equals(sentToken, sessionToken)) {
				c.setCsrfSafe(true);
			} else {
				c.setCsrfSafe(false);
				Sht.log(this).warning("CSRF detected.");
			}

		}
		return true;
	}

	/**
	 * セッションとMAVにトークンを格納しておく
	 */
	@Override
	public void postHandle(HttpServletRequest request,
			HttpServletResponse response, Object handler, ModelAndView mav)
			throws Exception {
		// リダイレクトする場合は不要
		if (mav != null) {
			if (!StringUtils.startsWith(mav.getViewName(), "redirect:")) {
				final String token = RandomStringUtils.randomAlphanumeric(128);
				request.getSession().setAttribute("secureToken", token);
				mav.addObject("secureToken", token);
			}
		}

		if (request.getMethod().equals("POST")
				&& handler instanceof IController) {
			// きちんとCSRFチェックが行われているかチェックする
			final IController c = (IController) handler;
			if (!c.isCsrfChecked()) {
				Sht.log(this).severe("CSRFチェックを行っていないPOST");
			}
			assert c.isCsrfChecked() : "CSRFチェックを行っていないPOST";
		}
	}

}
